/* TRL, 16.&17.&19.03.&11.04.&29.09.1996, 29.06.1998 */
/* TRL, 24.04.1999: Windows-Version                  */
/* TRL, 05.03.2000: Zeitzone                         */
/* TRL, 11.03.2000: konvertStammdat                  */
/* TRL, 08.11.2000: Parameter -Ddate                 */
/* TRL, 25.12.2001: KonvertBAV2002                   */
/* TRL, 24.03.2003: Vorlage aus kurzper-html.txt     */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#ifndef WIN32
#	include <unistd.h>
#	include <sys/time.h>
#	include <time.h>
#else
#	include <time.h>
#endif

#include <math.h>
#include <signal.h>

char copyright[] = "(c) Thorsten Lange, Bovenden, 24.05.2003\n    EMail: tl@thola.de\n    Home of BAV: http://www.bav-astro.de";

FILE *fileStern;
char fileSternName[256];
char fileHTMLvorlage[256];
char fileHTMLname[256];

double outTimezone;

void KonvertGCVS ()
{
  FILE *fCAT;
  char zeile[256];
  double periode,max,min,datum;
  
  fileStern = fopen(fileSternName,"w");
  fCAT = fopen("/dosc/astro/catalogs/gcvs/gcvs.dat","r");
  while (fgets(zeile,255,fCAT)!=0) {
    sscanf(zeile+115,"%lf",&periode);
    sscanf(zeile+93,"%lf",&datum);
    sscanf(zeile+73,"%lf",&max);
    sscanf(zeile+82,"%lf",&min);
    if ((    (zeile[64]=='E') 
         || ((zeile[64]=='R') && (zeile[65]=='R'))
/*         || ((zeile[64]=='D') && (zeile[65]=='C') && (zeile[66]=='E'))
           || ((zeile[64]=='C') && (zeile[65]=='E') && (zeile[66]=='P')) */
        ) /* Typ */
        && (max < 11) && (min-max > 0.5)
        && (periode > 0.3) && (datum > 0)
        && (atoi(zeile+22) > -1500) /* Deklination */
    ) {
      fprintf(fileStern,"%.12s%.6s%.7s%.9s %5.2f %5.2f %10.7f %10.7f\n",
                        zeile+7, zeile+16, zeile+22, zeile+64, 
                        max, min, periode, datum);
    }
  }
  fclose(fCAT);
  fclose(fileStern);
}

void KonvertBAV1996 ()
{
  FILE *fCAT;
  char zeile[256];
  double periode,max,min,datum;
  int i;
  
  fileStern = fopen(fileSternName,"w");
  fCAT = fopen("bkvss1996.txt","r");
  if (fCAT==NULL) {
  		fprintf(stderr,"File not found!\n");
  		return;
  }
  while (fgets(zeile,255,fCAT)!=0) {
    sscanf(zeile+43,"%lf",&periode);
    sscanf(zeile+54,"%lf",&datum);
    sscanf(zeile+31,"%lf",&max);
    sscanf(zeile+38,"%lf",&min);
    if ((    (zeile[21]=='E') 
         || ((zeile[21]=='R') && (zeile[22]=='R'))
        ) /* Typ */
    ) {
      for (i=43; i<52; i++)
        if (zeile[i]==' ') zeile[i]='0';
      fprintf(fileStern,"%.8s    %.6s%.7s%.9s %5.2f %5.2f %10.7f %10.7f\n",
                        zeile, zeile+8, zeile+14, zeile+21, 
                        max, min, periode, datum);
    }
  }
  fclose(fCAT);
  fclose(fileStern);
}

void KonvertBAV2002 ()
{
  FILE *fCAT;
  char zeile[256];
  double periode,max,min,datum;
  int i;
  
  fileStern = fopen(fileSternName,"w");
  fCAT = fopen("bkvss2002.txt","r");
  if (fCAT==NULL) 
  {
  		fprintf(stderr,"File not found!\n");
  		return;
  }
  while (fgets(zeile,255,fCAT)!=0) {
    sscanf(zeile+28,"%lf",&periode);
    sscanf(zeile+13,"%lf",&datum);
    datum += 2400000;
    sscanf(zeile+92,"%lf",&max);
    sscanf(zeile+97,"%lf",&min);
    if ((    (zeile[71]=='E') 
         || ((zeile[71]=='R') && (zeile[72]=='R'))
        ) /* Typ */
    ) {
      fprintf(fileStern,"%.8s    %.6s%.7s%.9s %5.2f %5.2f %10.7f %10.7f\n",
                        zeile, zeile+54, zeile+60, zeile+71, 
                        max, min, periode, datum);
    }
  }
  fclose(fCAT);
  fclose(fileStern);
}

void KonvertStammdat()
{
	FILE *fCAT;
	char zeile[256];
	double periode,max,min,datum,rect,decl;

	fileStern = fopen(fileSternName,"w");
	fCAT = fopen("stammdat.txt","r");
	while (fgets(zeile,255,fCAT)!=0) {
		if (zeile[0]!='#' && strlen(zeile)>20) {
			char *ptr = zeile+4;
			// search for second space
			while (*ptr!=' ')	ptr++;
			ptr++;	
			while (*ptr!=' ')	ptr++;
			sscanf(ptr+1, "%lf %lf %lf %lf %lf %lf", &rect, &decl, &datum, &periode, &max, &min );
			datum += 2400000;
			*ptr = '\0';	// cut at second space
			fprintf(fileStern,"%12.12s%02i    %c%02i              %5.2f %5.2f %10.7f %10.7f\n",
                           zeile+4, (int)rect, (decl<0 ? '-' : '+'), abs(decl),  
                           max, min, periode, datum);
		}
	}
	fclose(fCAT);
	fclose(fileStern);
}

#define mod1(f) (f-floor(f))

double Zeit2JulDat (char *datum)
{
  double jd,b;
  int t,m,j;

  for (t=0; t<strlen(datum); t++)
    if (datum[t]==' ') datum[t] = '0';
  t = atoi(datum);
  m = atoi(datum+3);
  j = atoi(datum+6);
  if (m<=2) { j--; m+=12; }
  if ((t<15) && (m<11) && (j<1583)) b = -2.0;
  else b = floor(j*0.0025)-floor(j*0.01);
  jd = floor(365.25*(double)j)+floor(30.6001*((double)m+1))+b+1720996.5+(double)t;
  return jd-outTimezone/24.0;
}


char *JulDat2Zeit (double jd, char *datum)
{
  double a,b,c,d,e,f,t,m,j,h;
  jd += outTimezone/24.0;
  jd += 0.000006; 
  a = floor(jd+0.5);
  if (a<2299161.0) c = a+1524.0;
  else {
    b = floor((a-1867216.25)/36524.25);
    c = a+b-floor(b*0.25)+1525.0;
  }
  d = floor((c-122.1)/365.25);
  e = floor(365.25*d);
  f = floor((c-e)/30.6001);
  t = c-e-floor(30.6001*f)+mod1(jd+0.5);
  m = floor(f-1.0-12.0*floor(f/14.0));
  j = d-4715.0-floor((7.000001+m)*0.1);
  h = mod1(t)*24+0.005;
  sprintf(datum, "%4i/%02i/%02i",(int)j,(int)m,(int)t);
  return datum;
}


double MakeTodayJD ()
{
  time_t sec;
  struct tm *tmSec; 
  char Datum[128],DatumStr[20];
  int monat;
  
  time(&sec);
  tmSec = localtime(&sec); 
  strcpy(Datum,asctime(tmSec));
  switch (Datum[4]) {
  case 'J': 
    if (Datum[5]=='a') monat=1; 
    else if (Datum[6]=='n') monat=6;
    else monat=7;
    break;
  case 'F': 
    monat=2; break;
  case 'M': 
    if (Datum[6]=='r') monat=3;
    else monat=5;
    break;
  case 'A': 
    if (Datum[5]=='p') monat=4; 
    else monat=8; 
    break;
  case 'S': 
    monat=9; break;
  case 'O': 
    monat=10; break;
  case 'N': 
    monat=11; break;
  case 'D': 
    monat=12; break;
  }  
  sprintf(DatumStr,"%.2s.%2i.%.4s",Datum+8,monat,Datum+20);
  return Zeit2JulDat(DatumStr);
}

void MakeHTML (double StartJD, int Tage)
{
	double testJD,periode;
	char 	zeile[128],datum[20],ok;
	int 	i,j, minN, minJD[100], rekt,zeit0,zeit1;
	FILE*  fVorlage;

	FILE *fileHTML = fopen(fileHTMLname,"w");

	/* HTML-Head */
	fVorlage = fopen(fileHTMLvorlage,"r");
	if (fVorlage==NULL)
	{
		fprintf(fileHTML,"<HTML>\n<HEAD>\n<TITLE>BAV - Kurzperiodische</TITLE>\n");
		fprintf(fileHTML,"</HEAD><BODY>\n");
		fprintf(fileHTML,"<H1>BAV - Kurzperiodische</H1>\n");
		fprintf(fileHTML,"<H2>Ephemeriden f&uuml;r die n&auml;chsten %i Tage</H2>\n",Tage);
		fprintf(fileHTML,"Die Minimazeitpunkte f&uuml;r Bedeckungsver&auml;nderliche\n");
		fprintf(fileHTML,"und Maximazeitpunkte f&uuml;r RR Lyr-Sterne sind bez&uuml;glich der\n");
		switch ((int)outTimezone) {
			case 0:	fprintf(fileHTML,"UT");	break;
			case 1:  fprintf(fileHTML,"MEZ (Mitteleurop&auml;ische Zeit)");			break;
			case 2:	fprintf(fileHTML,"MESZ (Mitteleurop&auml;ische Sommerzeit)"); 	break;
			default: fprintf(fileHTML,"UT%s%i", (outTimezone<0 ? "-" : "+"), abs((int)outTimezone) );
		}
		fprintf(fileHTML," in Stunden angegeben.\n");
		fprintf(fileHTML,"Falls gerade die Sommerzeit gilt, muss jeweils eine Stunde zu den angegeben Zeiten addiert werden.\n<BR><BR>\n");
	}
	else
	{
		int weiter = 1;
		while (weiter)
		{
			if (fgets(zeile,sizeof(zeile),fVorlage))
			{
				if (zeile[0]=='*')
					weiter = 0;
				else
					fprintf(fileHTML,zeile);
			}
			else
			{
				fclose(fVorlage);
				weiter = 0;
			}
		}
	}

	// Quelle Stammdat.txt:
	/*
	fprintf(fileHTML,"<UL>\n");
	fprintf(fileHTML,"<LI>* bedeutet Nebenminimum\n");
	fprintf(fileHTML,"<LI>\"Name\"A bedeutet Abstieg zum Minimum\n");
	fprintf(fileHTML,"<LI>\"Name\"B bedeutet Aufstieg vom Minimum (bei auergewhnlichem \"d\")\n");
	fprintf(fileHTML,"<LI>\"Name\"v bedeutet 0,25d nach dem GCVS-Minimum\n");
	fprintf(fileHTML,"<LI>\"Name\"z bedeutet 0,25d vor dem GCVS-Minimum\n");
	fprintf(fileHTML,"</UL>\n");
	*/
	
	/* Tabelle */
	fprintf(fileHTML,"<CENTER>\n");
	fprintf(fileHTML,"<table border=5 cellpadding=2 cellspacing=4>\n");
	fprintf(fileHTML,"<tr><th align=center>Stern</th><th align=center>Koordinaten 2000.0</th>\n");
	fprintf(fileHTML,"<th align=center>Typ</th>\n");
	fprintf(fileHTML,"<th align=center>Max / Min</th>\n");
	for (i=0; i<Tage; i++)
		fprintf(fileHTML,"<th align=center>%s</th>\n",JulDat2Zeit(StartJD+(double)i,datum));
	fprintf(fileHTML,"</tr>\n");

	fileStern = fopen(fileSternName,"r");
	JulDat2Zeit(StartJD,datum);
	rekt = 2*atoi(datum+5)-5;
	if (atoi(datum+8)>15) rekt++;
	if (rekt<0) rekt+=24;
	switch (atoi(datum+5)) {
		case 1:
		case 11:
		case 12: zeit0=9; zeit1=16; break;
		case 10:
		case 2: 	zeit0=8; zeit1=17; break;
		case 9:
		case 3: 	zeit0=7; zeit1=18; break;
		case 8:
		case 4: 	zeit0=6; zeit1=19; break;
		case 7:
		case 5: 	zeit0=5; zeit1=20; break;
		case 6: 	zeit0=5; zeit1=21; break;
	}	
	zeit0 = (zeit0*10);
	zeit1 = (zeit1*10);
	
  while (fgets(zeile,127,fileStern)!=0) {
    /* pruefen, ob Stern sichtbar */
		if ((zeile[18]==' ') && (zeile[19]==' ')  && (zeile[20]==' ')) {
			/* keine Position gegeben -> immer sichtbar */
			ok = 1;
		} else {
			sscanf(zeile+18,"%3i",&j);
			if      (j>45) j=24;
			else if (j>25) j=16;
			else if (j>12) j=12;
			else if (j>  0) j=8;
			else if (j>-10) j=6;
			else if (j>-20) j=4;
			else j=0;
			sscanf(zeile+12,"%2i",&i);
			if (rekt+j<24) 
				ok = ((rekt<=i) && (i<=rekt+j));
			else 
				ok = ((rekt<=i) || (i<=rekt+j-24));
		}
		if (ok) { /* Minima/Maxima berechnen */
			minN = 0;
			sscanf(zeile+46,"%lf %lf",&periode,&testJD);
			testJD = (StartJD-testJD)/periode;
			testJD = (1-mod1(testJD))*periode;
			if (testJD==periode) testJD = 0;
			while ((testJD < (double)Tage) && (minN<100)) {
				minJD[minN] = (int)(testJD*240+0.5);
				if ((minJD[minN] % 240 < zeit0) || (zeit1 < minJD[minN] % 240)) minN++;
				testJD += periode;
			}
		}
    if ((ok) && (minN>0)) {
      fprintf(fileHTML,"<tr><td align=left>%.12s</td>",zeile);
      fprintf(fileHTML,"<td align=left>%.13s</td>",zeile+12);
      fprintf(fileHTML,"<td align=center>%.2s</td>",zeile+25);
      sscanf(zeile+30,"%lf %lf",&periode,&testJD);
      fprintf(fileHTML,"<td align=center>%3.1f / %3.1f</td>",periode,testJD);
      j = 0;
      for (i=1; i<=Tage; i++) {
        ok = 0;
        fprintf(fileHTML,"<td align=center>");
        while ((j<minN) && (minJD[j]<240*i)) {
          if (ok>0) fprintf(fileHTML," / ");
          fprintf(fileHTML,"%2.1f",(double)(minJD[j] % 240)*0.1);
          ok++;
          j++;
        }
        fprintf(fileHTML,"</td>");
      }
      fprintf(fileHTML,"</tr>\n");
    } /* eof Stern in die Tabelle aufnehmen */
  } /* eof Stern untersuchen */

	fprintf(fileHTML,"</table>\n");
	fprintf(fileHTML,"</CENTER>\n");
	
	/* HTML-Tail */
	if (fVorlage)
	{
		while (fgets(zeile,sizeof(zeile),fVorlage))
		{
			fprintf(fileHTML,zeile);
		}
		fclose(fVorlage);
	}
	else
	{
		fprintf(fileHTML,"<HR size=3>\n<ADDRESS>\nThorsten Lange, Bovenden<BR>\n");
		fprintf(fileHTML,"EMail <A HREF=\"mailto:tl@thola.de\">tl@thola.de</A>\n");
		fprintf(fileHTML,"</BODY>\n</HTML>\n");
	}

	fclose(fileHTML);
}

void nix (int i)
{
}


#ifndef WIN32	// Linux

int main (int argc, char *argv[])
{
	double todayJD = 0;
	char pfad[256], outName[128];
	int schleife = 0;
	int days = 4;
	int i;

	pfad[0] = '\0';
	sprintf(fileSternName,"kurzper.dat");
	sprintf(outName,"bav_kurzper.html");
	outTimezone = 1.0;	/* MEZ */

	fprintf(stderr,"kurzper [-i?] [-n] [-z=<deltaH>] [-d=<days>] [-o=<outName>] [-D=dd.mm.yyyy] [<path>]\n");
	fprintf(stderr,"        deltaH: +1=MEZ, +2=MESZ, 0=UT\n");
	fprintf(stderr,"        -ig: include GCVS style data\n");
	fprintf(stderr,"        -ib: include bkvss style data\n");
	fprintf(stderr,"        -is: include stammdat style data\n");
	fprintf(stderr,copyright);
	fprintf(stderr,"\n");
	
	for (i=1; i<argc; i++) { 
		if (argv[i][0]=='-') {
			switch (argv[i][1]) {
				case 'i': 
					switch (argv[i][2]) {
					case 'g': KonvertGCVS();		break;
					case 'b': KonvertBAV2002();  	break;
					case 's': KonvertStammdat();	break;
					}
					break;
				case 'n': schleife = 1; 							break;
				case 'o': strcpy(outName,&argv[i][3]);			break;
				case 'z': outTimezone = atof(&argv[i][3]);	break;
				case 'd': days = atoi(&argv[i][3]);				break;
				case 'D': todayJD = Zeit2JulDat(&argv[i][3]);	break;
			}
		} else 
			strcpy(pfad,argv[i]);
	}

	sprintf(fileSternName,"%skurzper.dat",pfad);
	sprintf(fileHTMLvorlage,"%skurzper-html.txt",pfad);
	sprintf(fileHTMLname,"%s%s",pfad,outName);

	if (todayJD==0)
		todayJD = MakeTodayJD();
	MakeHTML(todayJD,days);
	fprintf(stderr,"%s erstellt!\n",fileHTMLname);

	while (schleife) {
		signal(SIGALRM,nix);
		alarm(3600*6);
		pause();
		if (MakeTodayJD()!=todayJD) {
			todayJD = MakeTodayJD();
			MakeHTML(todayJD,4);
		}
	} 
	
	return 0;
}

#else	// WIN32

#include "windows.h"

void Message( char* msg )
{
	char str[256];
	sprintf(str,"%s\n\n%s", msg, copyright );
	MessageBox(NULL,str,"kurzper",MB_OK);
}


int WINAPI WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow )
{
	char pfad[256], outName[128], *ptr;
	double todayJD = 0;
	pfad[0] = '\0';
	outTimezone = 1.0;	/* MEZ */
	sprintf(outName,"bav_kurzper.html");
	
	ptr = strstr(lpCmdLine,"-z=");
	if (ptr)	outTimezone = atof(ptr+3);

	ptr = strstr(lpCmdLine,"-o=");
	if (ptr)	{ 
		strncpy(outName,ptr+3,125);
		ptr = strchr(outName,' ');
		if (ptr)	*ptr = 0;
	}

	ptr = strstr(lpCmdLine,"-D=");
	if (ptr)	{ 
		todayJD = Zeit2JulDat(ptr+3);
	}
	
	ptr = strstr(lpCmdLine,"-h");
	if (ptr || strchr(lpCmdLine,'?')) {
		Message("kurzper [-z=<deltaH>] [-o=<outName.htm>] [-D=dd.mm.yyyy] [-h]");
		return 0;
	}
		
	sprintf(fileSternName,"%skurzper.dat",pfad);
	sprintf(fileHTMLname,"%s%s",pfad,outName);

	if (todayJD==0)
		todayJD = MakeTodayJD();
	MakeHTML(todayJD,4);

	strcat(fileHTMLname," erstellt!");
	Message(fileHTMLname);
	return 0;
}

#endif	// WIN32
